#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const path_1 = __importDefault(require("path"));
const dotenv_1 = __importDefault(require("dotenv"));
const utils_1 = require("./utils");
const fs_1 = __importDefault(require("fs"));
// const importDefault = (this && ()this as any).__importDefault) || function (mod: any) {
// return (mod && mod.__esModule) ? mod : { "default": mod };
// };
commander_1.program
    .arguments('[config]')
    .option('-p, --port <number>', 'the listening port', parseInt)
    .option('--https', 'start a https server for development', false)
    .option('--key <string>', 'the overlord graph key')
    .option('--production', 'specify production environment', false)
    .option('--envFile <string>', 'the .env file')
    .parse(process.argv);
const cwd = process.cwd();
let configFile = commander_1.program.args[0];
if (configFile) {
    configFile = path_1.default.resolve(cwd, configFile);
}
else {
    configFile = path_1.default.resolve(cwd, 'overlord.config.js');
    if (!fs_1.default.existsSync(configFile)) {
        configFile = path_1.default.resolve(cwd, 'overlord.config.ts');
        if (!fs_1.default.existsSync(configFile)) {
            console.error('You must specify the config file, or provide overlord.config.js or overlord.config.ts file.');
            process.exit(1);
        }
        else {
            try {
                require('ts-node/register');
            }
            catch (e) {
                console.error('Please install ts-node to run typescript config.');
                throw e;
            }
        }
    }
}
// const configFile = path.resolve(process.cwd(), program.args[0] || 'overlord.config.js');
utils_1.requireOrImport(configFile, (err, mod) => {
    if (err) {
        console.error(err);
        process.exit(1);
    }
    const config = __importDefault(mod).default;
    if (commander_1.program.port !== undefined)
        config.port = commander_1.program.port;
    if (commander_1.program.key)
        config.key = commander_1.program.key;
    config.dev = !commander_1.program.production;
    if (config.dev) {
        dotenv_1.default.config({ path: path_1.default.resolve(process.cwd(), commander_1.program.envFile || '.env') });
    }
    const { createServer } = require('./index');
    createServer(config);
});
