"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDevGatewayServer = void 0;
// import {ApolloServer} from 'apollo-server';
const apollo_server_express_1 = require("apollo-server-express");
const apollo_server_core_1 = require("apollo-server-core");
const gateway_1 = require("@apollo/gateway");
const legacyLoadServicesFromStorage_1 = require("@apollo/gateway/dist/legacyLoadServicesFromStorage");
const loadServicesFromRemoteEndpoint_1 = require("@apollo/gateway/dist/loadServicesFromRemoteEndpoint");
const RemoteGraphQLDataSource_1 = require("./datasources/RemoteGraphQLDataSource");
const utils_1 = require("./utils");
const express_1 = __importDefault(require("express"));
const https_1 = __importDefault(require("https"));
const http_1 = __importDefault(require("http"));
const mkcert_1 = __importDefault(require("mkcert"));
function createDevGatewayServer(url, port, serviceName, key, studioUrl, gatewayUrl, httpsOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const localServiceList = yield getLocalServiceList(url);
        let studioServiceList = [];
        if (serviceName && key) {
            studioServiceList = yield getStudioServiceList(serviceName, key, gatewayUrl);
        }
        const gateway = new gateway_1.ApolloGateway({
            debug: true,
            localServiceList: [...localServiceList, ...studioServiceList],
            buildService({ url }) {
                return new RemoteGraphQLDataSource_1.RemoteGraphlQLDataSource({ url });
            }
        });
        const server = new apollo_server_express_1.ApolloServer({
            apollo: { key },
            gateway,
            subscriptions: false,
            context({ req, res }) {
                return { req, res };
            },
            playground: true,
            plugins: key ? [
                apollo_server_core_1.ApolloServerPluginUsageReporting({
                    endpointUrl: studioUrl,
                    sendHeaders: {
                        all: true
                    },
                    sendVariableValues: {
                        all: true,
                    },
                }),
            ] : undefined,
        });
        yield server.start();
        const app = express_1.default();
        server.applyMiddleware({
            app,
            path: '/',
            cors: {
                origin(origin, callback) {
                    // callback(null, !!origin && /\.ksyun\.com(:\d+)?$/.test(origin));
                    callback(null, true);
                },
                credentials: true,
            }
        });
        let httpServer;
        if (httpsOptions === true) {
            const { key, cert } = yield genCert();
            httpServer = https_1.default.createServer({ cert, key }, app);
        }
        else if (httpsOptions) {
            httpServer = https_1.default.createServer(httpsOptions, app);
        }
        else {
            httpServer = http_1.default.createServer(app);
        }
        httpServer.listen({ port }, () => {
            console.log(`🚀 Gateway server ready at ${httpsOptions ? 'https' : 'http'}://localhost:${port}`);
        });
    });
}
exports.createDevGatewayServer = createDevGatewayServer;
function getStudioServiceList(serviceName, key, gatewayUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const fetcher = gateway_1.getDefaultGcsFetcher();
        const apiKeyHash = utils_1.createSHA('sha512').update(key).digest('hex');
        const graphId = key.split(':', 2)[1];
        const { serviceDefinitions } = yield legacyLoadServicesFromStorage_1.getServiceDefinitionsFromStorage({
            graphId,
            apiKeyHash,
            graphVariant: 'current',
            federationVersion: 1,
            fetcher,
        });
        const serviceList = [];
        if (serviceDefinitions) {
            serviceDefinitions.map(({ name, url, typeDefs }) => {
                if (name === serviceName)
                    return;
                serviceList.push({
                    name,
                    url: gatewayUrl + '/' + graphId,
                    typeDefs
                });
            });
        }
        return serviceList;
    });
}
function getLocalServiceList(url) {
    return __awaiter(this, void 0, void 0, function* () {
        const dataSource = new gateway_1.RemoteGraphQLDataSource({ url });
        const serviceList = [{ name: '__local__', url, dataSource }];
        const { serviceDefinitions } = yield loadServicesFromRemoteEndpoint_1.getServiceDefinitionsFromRemoteEndpoint({
            serviceList,
            getServiceIntrospectionHeaders: () => __awaiter(this, void 0, void 0, function* () { return undefined; }),
            serviceSdlCache: new Map(),
        });
        return serviceDefinitions || [];
    });
}
function genCert() {
    return __awaiter(this, void 0, void 0, function* () {
        const ca = yield mkcert_1.default.createCA({
            organization: 'Overlord CA',
            countryCode: 'ZH',
            state: 'BeiJing',
            locality: 'Haidian',
            validityDays: 365,
        });
        return yield mkcert_1.default.createCert({
            domains: ['127.0.0.1', 'localhost', '::1', 'fe80::1'],
            validityDays: 365,
            caKey: ca.key,
            caCert: ca.cert,
        });
    });
}
