"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = void 0;
const apollo_server_1 = require("apollo-server");
const federation_1 = require("@apollo/federation");
const apollo_server_core_1 = require("apollo-server-core");
const utils_1 = require("./utils");
const defaultSchema_1 = require("./defaultSchema");
function createServer(_a) {
    var { schema = defaultSchema_1.schema, port = 8090, dev = true, key, serviceName, plugins, https } = _a, rest = __rest(_a, ["schema", "port", "dev", "key", "serviceName", "plugins", "https"]);
    return __awaiter(this, void 0, void 0, function* () {
        const app = new apollo_server_1.ApolloServer(Object.assign({ schema: federation_1.buildFederatedSchema(schema), context({ req, res }) {
                return { req, res };
            }, plugins: [
                // disable always, because we maybe in development environment
                apollo_server_core_1.ApolloServerPluginUsageReportingDisabled(),
                ...(plugins || []),
            ] }, rest));
        const { url } = yield app.listen(dev ? 0 : port);
        console.log(`🚀 App server ready at ${url}`);
        if (dev) {
            // we must initialize environment variables before importing devSever
            if (!key) {
                key = process.env.OVERLORD_KEY || process.env.APOLLO_KEY;
            }
            if (!key) {
                console.log(`
                \x1b[33mYou have not specify the environment variable: OVERLORD_KEY or the 'key'
                field in 'overlod.config.js', so we will start a server without federation.
                \x1b[0m
            `);
                // throw new Error(`You must specify the environment variable: OVERLORD_KEY` + 
                // ` or the 'key' field in overlord.config.js.`);
            }
            // find the studio that exists the key
            yield Promise.all(key ? Object.values(urlMap).map(item => {
                return utils_1.request(`${item.studio}/api/key?key=${encodeURIComponent(key)}`);
            }) : [[true, false, false]]).then(([dev, preview, production]) => {
                if (dev) {
                    return initEnvVariables('development');
                }
                else if (preview) {
                    return initEnvVariables('preview');
                }
                else if (production) {
                    return initEnvVariables('production');
                }
                else {
                    throw new Error('Can not find the key in any environment, please check it.');
                }
            }).then(({ studio, gateway }) => __awaiter(this, void 0, void 0, function* () {
                const { createDevGatewayServer } = yield Promise.resolve().then(() => __importStar(require('./devServer')));
                yield createDevGatewayServer(url, port, serviceName, key, studio, gateway, https);
            }));
        }
        return app;
    });
}
exports.createServer = createServer;
const urlMap = {
    development: {
        studio: 'http://dev.studio.overlord.ksyun.com',
        gateway: 'http://overlord-dev-api.ksyun.com',
    },
    preview: {
        studio: 'http://preview.studio.overlord.ksyun.com',
        gateway: 'https://overlord-preview-api.ksyun.com',
    },
    production: {
        studio: 'http://studio.overlord.ksyun.com',
        gateway: 'https://overlord-api.ksyun.com',
    }
};
function initEnvVariables(env) {
    console.log(`\x1b[32mUse ${env} environment gateway.\x1b[0m`);
    const { studio, gateway } = urlMap[env];
    process.env.APOLLO_STORAGE_SECRET_BASE_URL = studio;
    process.env.APOLLO_PARTIAL_SCHEMA_BASE_URL = studio;
    return { studio, gateway };
}
