"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.requireOrImport = exports.request = exports.createSHA = exports.setHeaders = void 0;
const crypto_1 = __importDefault(require("crypto"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const url_1 = require("url");
function setHeaders(req, headers) {
    for (let key in req.headers) {
        const value = req.headers[key];
        const lowerKey = key.toLowerCase();
        if (typeof value === 'string' &&
            (lowerKey === 'cookie' ||
                lowerKey.substr(0, 2) === 'x-')) {
            headers.set(key, value);
        }
    }
}
exports.setHeaders = setHeaders;
function createSHA(kind) {
    return crypto_1.default.createHash(kind);
}
exports.createSHA = createSHA;
function request(url) {
    return new Promise((resolve, reject) => {
        const hp = /^https:\/\//.test(url) ? https_1.default : http_1.default;
        hp.get(url, res => {
            const statusCode = res.statusCode;
            if (statusCode && statusCode >= 200 && statusCode < 400) {
                let data = '';
                res.on('data', chunk => data += chunk);
                res.on('end', () => {
                    try {
                        resolve(JSON.parse(data));
                    }
                    catch (e) {
                        resolve(false);
                    }
                });
            }
            else {
                resolve(false);
            }
        }).on('error', err => {
            resolve(false);
        });
    });
}
exports.request = request;
// @reference: https://github.com/gulpjs/gulp-cli/blob/c9e9125c5e95b213dafb53cd9717c70bf66a47a9/lib/shared/require-or-import.js 
let importESM = null;
try {
    // Node.js <10 errors out with a SyntaxError when loading a script that uses import().
    // So a function is dynamically created to catch the SyntaxError at runtime instead of parsetime.
    // That way we can keep supporting all Node.js versions all the way back to 0.10.
    importESM = new Function('id', 'return import(id);');
}
catch (e) {
    importESM = null;
}
function requireOrImport(path, callback) {
    let err = null;
    let cjs;
    try {
        cjs = require(path);
    }
    catch (e) {
        if (url_1.pathToFileURL && importESM && e.code === 'ERR_REQUIRE_ESM') {
            const url = url_1.pathToFileURL(path);
            importESM(url).then((esm) => {
                callback(null, esm.default);
            }, callback);
            return;
        }
        err = e;
    }
    process.nextTick(() => callback(err, cjs));
}
exports.requireOrImport = requireOrImport;
